package sesame.tutorial;

import java.io.File;
import java.net.URL;
import java.util.List;

import org.openrdf.OpenRDFException;
import org.openrdf.model.Statement;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.ValueFactory;
import org.openrdf.query.BindingSet;
import org.openrdf.query.GraphQueryResult;
import org.openrdf.query.Query;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.QueryResult;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.TupleQueryResult;
import org.openrdf.query.resultio.sparqlxml.SPARQLResultsXMLWriter;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.RepositoryException;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.rio.RDFFormat;
import org.openrdf.sail.inferencer.fc.ForwardChainingRDFSInferencer;
import org.openrdf.sail.memory.MemoryStore;

public class TestData {

	static final String inputDataFileName  = "/home/kkyzir/Desktop/SPARQL tutorial/rdf-data/culture_data.rdf";
	
	
	public static void main(String[] args) {
		// Sesame RDF API: http://www.openrdf.org/doc/sesame2/api/

		try {
			//Create a new main memory repository 
			MemoryStore store = new MemoryStore();
			Repository repo = new SailRepository(store);
			repo.initialize();

			//Store file
			try {
				File file = new File(inputDataFileName);
				String fileBaseURI = "http://www.di.uoa.gr/~pms547/culture#";
				RDFFormat fileRDFFormat = RDFFormat.RDFXML;
				
				RepositoryConnection con = repo.getConnection();
				try {
					//store the file
					con.add(file, fileBaseURI, fileRDFFormat);
				}
				finally {
					con.close();
				}
			}
			catch (OpenRDFException e) {
				e.printStackTrace();
			}
			catch (java.io.IOException e) {
				// handle io exception
				e.printStackTrace();
			}


			//To sesame ypostirizei:
			//Tuple queries: queries that produce sets of value tuples.
			//Graph queries: queries that produce RDF graphs
			//Boolean queries: true/false queries			
			
			//Evaluate a SPARQL tuple query
			try {
				RepositoryConnection con = repo.getConnection();
				try {
					String queryString1 = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?x ?y " +
			        " WHERE { ?x  ns:creates  ?y } ";
	        
			        String queryString2 = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?x ?y " +
			        " WHERE { ?x  ns:paints  ?y . " +
			        "   ?y ns:exhibited <http://www.louvre.fr/> .} ";
			        
			        String queryString3 = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?x " +
			        " WHERE { ?x  ns:first_name  \"August\" . } ";
			        
			        String queryString3b = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?y " +
			        " WHERE { ?x  ns:first_name  \"August\" ." +
			        "	?x ns:last_name ?y . } ";
			        
			        String queryString4 = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?x " +
			        " WHERE { ?x  ns:first_name  ?y" +
			        " FILTER regex(str(?x), \"rodin\") } ";
			        
			        String queryString5 = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?x ?y " +
			        " WHERE { ?x  ns:created  ?y " +
			        " FILTER (?y < 1990) } ";
			        
			        String queryString6 = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?x ?z ?w " +
			        " WHERE { ?x   ns:paints  ?y ." +
			        "					 " +
			        " OPTIONAL {?x ns:first_name ?z ." +
			        "			?x ns:last_name ?w}} ";
			        
			        String queryString7 = "prefix ns:   <http://www.di.uoa.gr/~pms547/culture#>" +
			        " SELECT ?x ?y " +
			        " WHERE { {?x  ns:sculpts  ?y }" +
			        " UNION {?x ns:paints ?y}  } ";
					
			        String queryString = queryString5;
					TupleQuery tupleQuery = con.prepareTupleQuery(QueryLanguage.SPARQL, queryString);
					TupleQueryResult result = tupleQuery.evaluate();
					System.out.println("Query:\n" + queryString);
					try {
						//iterate the result set
						//result = tupleQuery.evaluate();
						while (result.hasNext()) {
							BindingSet bindingSet = result.next();
							System.out.println(bindingSet.toString());
//							Value valueOfX = bindingSet.getValue("x");
//							Value valueOfY = bindingSet.getValue("y");
//
//							System.out.println("?x="  + valueOfX +  " ?y=" + valueOfY);

						}

						/*//iterate #2
						result = tupleQuery.evaluate();
						List<String> bindingNames = result.getBindingNames();
						while (result.hasNext()) {
							BindingSet bindingSet = result.next();
							Value firstValue = bindingSet.getValue(bindingNames.get(0));
							Value secondValue = bindingSet.getValue(bindingNames.get(1));
							Value thirdValue = bindingSet.getValue(bindingNames.get(2));

							System.out.println("?x=" + firstValue + ", ?p=" + secondValue + ", ?y=" + thirdValue);
						}*/

						/*//iterate #3
						SPARQLResultsXMLWriter sparqlWriter = new SPARQLResultsXMLWriter(System.out);
						tupleQuery.evaluate(sparqlWriter);*/
					}
					finally {
						result.close();
					}
				}
				catch (Exception e) {
					//handle exception
					e.printStackTrace();
				} finally {
					con.close();
				}
			}
			catch (OpenRDFException e) {
				// handle exception
				e.printStackTrace();
			}

			/*//Evaluate a query that produces an RDF graph
			try {
				RepositoryConnection con = repo.getConnection();
				try {
					GraphQueryResult graphResult = con.prepareGraphQuery(QueryLanguage.SPARQL, 
							"PREFIX rdf:   <http://www.w3.org/1999/02/22-rdf-syntax-ns#> " +
							"CONSTRUCT {_:v rdf:type rdf:Statement; rdf:Subject ?x ; rdf:Predicate ?p; rdf:Object ?y} " +
							"WHERE {?x ?p ?y . }").evaluate();
				
					while (graphResult.hasNext()) {
						Statement st = graphResult.next();
						System.out.println(st.toString());
					}
				}
				finally {
					con.close();
				}
			}
			catch (OpenRDFException e) {
				// handle exception
				e.printStackTrace();
			}*/
		} catch (RepositoryException e) {
			// handle exception
			e.printStackTrace();
		}
	}
}
